% file: run_HMR-STAR-50window-n1000-H2.m  
% with test statistic T ~ N(0,1): equal pdf under H0
% Test results in map:
% c:\\ ...
%
% Coded by: JDG
%
% Note: adjust all "save" statements to a pre-defined 
%       map location on your PC, e.g.,
%   save('c:\Hidden-Markov-KDE-HMM\ar_kde\SETAR100_ALL\matfile')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all; clc;

datastr = ' ...';
warning('off','all')


default_opts3;  % with maxorderm = 5, maxorderh = 3, maxstates = 3

ntrain = 1000;   % training  KEEP it FIXED
ntest  = 1000;   % validation KEEP it FIXED
nr     = ntrain+ntest;
low    = 2;
high   = low;
s      = low;

hzon   = 1;              % forecast horizon, CHANGE  
hmax   = 2;              % maximum forecast horizon
window = 49;  % 100;     % (ntrain-ntest-hmax+1);
% 500-450-2+1=49 rolling "hzon=1 or 2" forecasts


%% HMR-STAR(2,2) parameters 
mu      = [-1.0, 1.0];
theta   = [-0.8, 0.8];
sigma   = [2.0, 1.0];
delta   = 5;
r       = 0;

prob_tran = [.6, .4; .3, .7]; % transition probabilities
d         = 1;       % delay
n         = 5000;    % total sample size, 1000 for training
nr        = 1000;    % number of observations in training set
warmup    = 3500;
y         = zeros(1,n+warmup+d);
x         = zeros(1,n+warmup+d);
error     = zeros(1,n);
%%  end HMR-STAR parameters

crit  = 1.96;   % 5% critical value T ~ N(0,1), 2-sided
crit2 = 1.645;  % 5% one-sided

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%% HMR-STAR(2,2)
x(1)  = 1;
for i = 1:(n+warmup+d)
  prob = prob_tran(1,x(i));
  if (unifrnd(0,1)< prob) 
    x(i+1) = 1;  
  else  
    x(i+1) = 2;
  end 
end

for (i =(1+warmup+d):(n+warmup+d))
   error(i) = normrnd(0,1);
   y(i)     = mu(x(i))+theta(x(i))*normcdf((y(i-d)-r)/delta)+sigma(x(i))*error(i);
end

yy = y((1:n)+warmup+d);
x  = x((1:n)+warmup+d);  % series with states 1 and 2

clps4_n = zeros(window,1);
clps3_n = zeros(window,1);
clps2_n = zeros(window,1);
clps1_n = zeros(window,1);

for uu = 1:window  % Start rolling forecasting loop
rng('shuffle')
  value = [s  uu];
  disp('step (s) and  uu = ')
  disp(value)
  
  xt = yy(uu+ntrain:(ntrain+uu+ntrain-1));    % training 
  xf = yy(ntrain+ntrain+uu+hzon-1);           % forecast values
  xv = yy(ntrain+ntrain+uu:(ntrain+ntrain+uu+nr-1));
            
  xmin   = min(xt)-10;
  xmax   = max(xt)+10;
  xpts   = linspace(xmin,xmax,ntrain);
  cond   = xt;   % condition on the 500 points of the set xs, rolling

  maxima = estimate_discrete_maxima(xt);  % obtain maxima
  maxima = maxima(xt(maxima) > 50); % Extract non-noise peaks, CHANGE 50

  gammas = phaseinit(datastr,ntrain,maxstates,maxima);

 kdehmm_experiment5(datastr,ntrain,ntest,gridres,condq,maxorderm,maxorderh,...
   maxstates,maxiter,nreinits,inititer,miniter,minstep,gammas,...
   hmmgammainit,xv,xt);    

% Compute conditional pdf at xpts and xf  
  load('matfile.mat')
  model1      = mmpts(maxorderm+1,1);         
  clps1_n(uu) = condlogprob(model1,cond,xf);      % stationary KDE-MM
  
  model2      = hmmt(end);          
  clps2_n(uu) = condlogprob(model2,cond,xf);      % 

  model3      = art(1,2);        
  clps3_n(uu) = condlogprob(model3,cond,xf);      % AR-HMM

  model4      = kdeht(1,1);         
  clps4_n(uu) = condlogprob(model4,cond,xf);      % KDE-HMM
    
  dif_12(uu) = clps1_n(uu) - clps2_n(uu); % KDE_MM vs G-HMM
  dif_13(uu) = clps1_n(uu) - clps3_n(uu); % KDE_MM vs AR-HMM
  dif_14(uu) = clps1_n(uu) - clps4_n(uu); % KDE_MM vs KDE-HMM
  dif_23(uu) = clps2_n(uu) - clps3_n(uu); % G-MM   vs AR-HMM
  dif_24(uu) = clps2_n(uu) - clps4_n(uu); % G_MM   vs KDE-HMM
  dif_34(uu) = clps3_n(uu) - clps4_n(uu); % AR_HMM vs KDE-HMM
  
  delta1(uu) = (xf - clps1_n(uu))^2;
  delta2(uu) = (xf - clps2_n(uu))^2;  
  delta3(uu) = (xf - clps3_n(uu))^2;
  delta4(uu) = (xf - clps4_n(uu))^2;  
  
  forecast(uu) = xf;  % forecast values

end  % end rolling forecasting loop uu=1:window

m_12 = mean(dif_12); 
m_13 = mean(dif_13);
m_14 = mean(dif_14);
m_23 = mean(dif_23);
m_24 = mean(dif_24);
m_34 = mean(dif_34);

s_12 = std(dif_12); 
s_13 = std(dif_13);
s_14 = std(dif_14);
s_23 = std(dif_23);
s_24 = std(dif_24);
s_34 = std(dif_34);

%%%  Amisano and Giacomini LR-type test %%%%%
T12 = sqrt(window) * m_12 / std(dif_12);
T13 = sqrt(window) * m_13 / std(dif_13);
T14 = sqrt(window) * m_14 / std(dif_14);
T23 = sqrt(window) * m_23 / std(dif_23);
T24 = sqrt(window) * m_24 / std(dif_24);
T34 = sqrt(window) * m_34 / std(dif_34);

%% calculate the frequency of rejecting H0: equal performance of pdfs
rej12 = 0;
rej13 = 0;
rej14 = 0;
rej23 = 0;
rej24 = 0;
rej34 = 0;

if (T12 <= -crit || T12 >= crit) % KDE_MM vs G-HMM, || = OR
  rej12 = rej12 + 1;
end

if T12<0;
    sign_T12 ='(-)';
elseif  T12 >0;
    sign_T12 ='(+)';
end;

if (T13<= -crit || T13 >= crit) % KDE_MM vs AR-HMM
  rej13 = rej13 + 1;
end

if T13<0;
    sign_T13 ='(-)';
elseif  T13 >0;
    sign_T13 ='(+)';
end;

if (T14 <= -crit || T14 >= crit) % KDE_MM vs KDE-HMM
  rej14 = rej14 + 1;
end

if T14<0;
    sign_T14 ='(-)';
elseif  T14 >0;
    sign_T14 ='(+)';
end

if (T23 <= -crit || T23 >= crit) % G_MM vs AR-HMM
  rej23 = rej23 + 1;
end

if T23<0;
    sign_T23 ='(-)';
elseif  T23 >0;
    sign_T23 ='(+)';
end;

if (T24 <= -crit || T24 >= crit) % G_MM vs KDE-HMM
  rej24 = rej24 + 1;
end

if T24<0;
    sign_T24 ='(-)';
elseif  T24 >0;
    sign_T24 ='(+)';
end

if (T34 <= -crit || T34 >= crit) % AR_HMM vs KDE-HMM
  rej34 = rej34 + 1;
end

if T34<0;
    sign_T34 ='(-)';
elseif  T34 >0;
    sign_T34 ='(+)';
end

%%%%%%%%%%%%%%% crit2 = 1.645 %%%%%%%%%%%%%%%%%%
rej12_2 = 0;
rej13_2 = 0;
rej14_2 = 0;
rej23_2 = 0;
rej24_2 = 0;
rej34_2 = 0;

if (T12 <= -crit2 || T12 >= crit2) % KDE_MM vs G-HMM, || = OR
  rej12_2 = rej12_2 + 1;
end

if T12<0;
    sign_T12_2 ='(-)';
elseif  T12 >0;
    sign_T12_2 ='(+)';
end;

if (T13 <= -crit2 || T13 >= crit2) % KDE_MM vs AR-HMM
  rej13_2 = rej13_2 + 1;
end

if T13<0;
    sign_T13_2 ='(-)';
elseif  T13 >0;
    sign_T13_2 ='(+)';
end;

if (T14 <= -crit2 || T14 >= crit2) % KDE_MM vs KDE-HMM
  rej14_2 = rej14_2 + 1;
end

if T14<0;
    sign_T14_2 ='(-)';
elseif  T14 >0;
    sign_T14_2 ='(+)';
end;

if (T23 <= -crit2 || T23 >= crit2) % G_MM vs AR-HMM
  rej23_2 = rej23_2 + 1;
end

if T23<0;
    sign_T23_2 ='(-)';
elseif  T23 >0;
    sign_T23_2 ='(+)';
end;

if (T24 <= -crit2 || T24 >= crit2) % G_MM vs KDE-HMM
  rej24_2 = rej24_2 + 1;
end

if T24<0;
    sign_T24_2 ='(-)';
elseif  T24 >0;
    sign_T24_2 ='(+)';
end;

if (T34 <= -crit2 || T34 >= crit2) % AR_HMM vs KDE-HMM
  rej34_2 = rej34_2 + 1;
end

if T34<0;
    sign_T34_2 ='(-)';
elseif  T34 >0;
    sign_T34_2 ='(+)';
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% CPA test statistic value (Giacomini - White)  Wald-type test
% reject H0: E( Loss(model A) - Loss(model B) )= 0. 
% The sign of the test-statistic
% indicates which forecast performs better:
% a + (positive) test-statistic indicates
% that model A forecast produces larger average loss than 
% the model B forecast 
% (model B outperforms model A), while 
% a - (negative) sign indicates the opposite.

[CPA_pv_12 sign12] = CPAtest2(delta1', delta2'); % KDE_MM vs G-HMM 
[CPA_pv_13 sign13] = CPAtest2(delta1', delta3'); % KDE_MM vs AR-HMM  (table) 
[CPA_pv_14 sign14] = CPAtest2(delta1', delta4'); % KDE_MM vs KDE_HMM (table)
[CPA_pv_23 sign23] = CPAtest2(delta2', delta3'); % G-HMM  vs AR-HMM
[CPA_pv_24 sign24] = CPAtest2(delta2', delta4'); % G-HMM  vs KDE-HMM 
[CPA_pv_34 sign34] = CPAtest2(delta3', delta4'); % AR-HMM vs KDE-HMM (table)
   

%%%%%%%%%%%%%%%%%% OUTPUT %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

disp(' %%%%%%%%%%%%%%%%%% 1.96 %%%%%%%%%%%%%%%%%%%%%%%%% ')
disp('crit = 1.96 rej12 rej 13 rej14 rej3 rej24 rej 34 ')
rejection =[rej12 rej13 rej14 rej23 rej24 rej34];
disp(rejection)

sign_AW = [sign_T12 sign_T13 sign_T14 sign_T23 sign_T24 sign_T34];
disp('sign-Amisano-Giacomini')
disp(sign_AW)

disp(' ')
disp(' %%%%%%%%%%%%%%%%%% 1.645 %%%%%%%%%%%%%%%%%%%%%%%%% ')

disp(' crit2 = 1.645 rej12_2 rej 13_s rej14_2 rej23_2 rej24_2 rej34_2 ')
rejection_2 =[rej12_2 rej13_2 rej14_2 rej23_2 rej24_2 rej34_2];
disp(rejection_2)

sign_AW_2 = [sign_T12_2 sign_T13_2 sign_T14_2 sign_T23_2 sign_T24_2 sign_T34_2];
disp('crit2 = 1.645 sign-Amisano-Giacomini')
disp(sign_AW_2)

disp(' ')
disp(' %%%%%%%%%%%%%%%%%% CPA %%%%%%%%%%%%%%%%%%%%%%%%% ')

CPA=[CPA_pv_12 CPA_pv_13 CPA_pv_14 CPA_pv_23 CPA_pv_24 CPA_pv_34];
disp('p-values: CPA12 CPA13 CPA14 CPA23 CPA24 CPA34 ')
disp(CPA)

sign_GW = [sign12 sign13 sign14 sign23 sign24 sign34];
disp('sign-Giacomini-White')
disp(sign_GW)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%% Original %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
prob1 = exp(clps1_n);
prob2 = exp(clps2_n);
prob3 = exp(clps3_n);
prob4 = exp(clps4_n);

%%%%%%%%%%%%%%%%%%%%  END MAIN %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
